@@ -20,6 +20,7 @@
 
 """Utility methods for working with WSGI servers."""
 
+import socket
 import sys
 
 import eventlet.wsgi
@@ -28,14 +29,24 @@ import ssl
 import webob.dec
 import webob.exc
 
+from keystone.common import config
 from keystone.common import logging
 from keystone.common import utils
 from keystone import exception
+from keystone.openstack.common import importutils
 from keystone.openstack.common import jsonutils
 
 
+CONF = config.CONF
 LOG = logging.getLogger(__name__)
 
+# Environment variable used to pass the request context
+CONTEXT_ENV = 'openstack.context'
+
+
+# Environment variable used to pass the request params
+PARAMS_ENV = 'openstack.params'
+
 
 class WritableLogger(object):
     """A thin wrapper that responds to `write` and logs."""
@@ -63,27 +74,39 @@ class Server(object):
 
     def start(self, key=None, backlog=128):
         """Run a WSGI server with the given application."""
-        LOG.debug('Starting %(arg0)s on %(host)s:%(port)s' %
+        LOG.debug(_('Starting %(arg0)s on %(host)s:%(port)s') %
                   {'arg0': sys.argv[0],
                    'host': self.host,
                    'port': self.port})
-        socket = eventlet.listen((self.host, self.port), backlog=backlog)
+
+        # TODO(dims): eventlet's green dns/socket module does not actually
+        # support IPv6 in getaddrinfo(). We need to get around this in the
+        # future or monitor upstream for a fix
+        info = socket.getaddrinfo(self.host,
+                                  self.port,
+                                  socket.AF_UNSPEC,
+                                  socket.SOCK_STREAM)[0]
+        _socket = eventlet.listen(info[-1],
+                                  family=info[0],
+                                  backlog=backlog)
         if key:
-            self.socket_info[key] = socket.getsockname()
+            self.socket_info[key] = _socket.getsockname()
         # SSL is enabled
         if self.do_ssl:
             if self.cert_required:
                 cert_reqs = ssl.CERT_REQUIRED
             else:
                 cert_reqs = ssl.CERT_NONE
-            sslsocket = eventlet.wrap_ssl(socket, certfile=self.certfile,
+            sslsocket = eventlet.wrap_ssl(_socket, certfile=self.certfile,
                                           keyfile=self.keyfile,
                                           server_side=True,
                                           cert_reqs=cert_reqs,
                                           ca_certs=self.ca_certs)
-            socket = sslsocket
+            _socket = sslsocket
 
-        self.greenthread = self.pool.spawn(self._run, self.application, socket)
+        self.greenthread = self.pool.spawn(self._run,
+                                           self.application,
+                                           _socket)
 
     def set_ssl(self, certfile, keyfile=None, ca_certs=None,
                 cert_required=True):
@@ -107,8 +130,12 @@ class Server(object):
     def _run(self, application, socket):
         """Start a WSGI server in a new green thread."""
         log = logging.getLogger('eventlet.wsgi.server')
-        eventlet.wsgi.server(socket, application, custom_pool=self.pool,
-                             log=WritableLogger(log))
+        try:
+            eventlet.wsgi.server(socket, application, custom_pool=self.pool,
+                                 log=WritableLogger(log))
+        except Exception:
+            LOG.exception(_('Server error'))
+            raise
 
 
 class Request(webob.Request):
@@ -186,12 +213,17 @@ class Application(BaseApplication):
         arg_dict = req.environ['wsgiorg.routing_args'][1]
         action = arg_dict.pop('action')
         del arg_dict['controller']
-        LOG.debug('arg_dict: %s', arg_dict)
+        LOG.debug(_('arg_dict: %s'), arg_dict)
 
         # allow middleware up the stack to provide context & params
-        context = req.environ.get('openstack.context', {})
+        context = req.environ.get(CONTEXT_ENV, {})
         context['query_string'] = dict(req.params.iteritems())
-        params = req.environ.get('openstack.params', {})
+        context['path'] = req.environ['PATH_INFO']
+        params = req.environ.get(PARAMS_ENV, {})
+        if 'REMOTE_USER' in req.environ:
+            context['REMOTE_USER'] = req.environ['REMOTE_USER']
+        elif context.get('REMOTE_USER', None) is not None:
+            del context['REMOTE_USER']
         params.update(arg_dict)
 
         # TODO(termie): do some basic normalization on methods
@@ -203,12 +235,15 @@ class Application(BaseApplication):
         try:
             result = method(context, **params)
         except exception.Unauthorized as e:
-            LOG.warning("Authorization failed. %s from %s"
+            LOG.warning(_("Authorization failed. %s from %s")
                         % (e, req.environ['REMOTE_ADDR']))
             return render_exception(e)
         except exception.Error as e:
             LOG.warning(e)
             return render_exception(e)
+        except TypeError as e:
+            logging.exception(e)
+            return render_exception(exception.ValidationError(e))
         except Exception as e:
             logging.exception(e)
             return render_exception(exception.UnexpectedError(exception=e))
@@ -221,7 +256,17 @@ class Application(BaseApplication):
             return result
         elif isinstance(result, webob.exc.WSGIHTTPException):
             return result
-        return render_response(body=result)
+
+        response_code = self._get_response_code(req)
+        return render_response(body=result, status=response_code)
+
+    def _get_response_code(self, req):
+        req_method = req.environ['REQUEST_METHOD']
+        controller = importutils.import_class('keystone.common.controller')
+        code = None
+        if isinstance(self, controller.V3Controller) and req_method == 'POST':
+            code = (201, 'Created')
+        return code
 
     def _normalize_arg(self, arg):
         return str(arg).replace(':', '_').replace('-', '_')
@@ -235,8 +280,8 @@ class Application(BaseApplication):
             try:
                 user_token_ref = self.token_api.get_token(
                     context=context, token_id=context['token_id'])
-            except exception.TokenNotFound:
-                raise exception.Unauthorized()
+            except exception.TokenNotFound as e:
+                raise exception.Unauthorized(e)
 
             creds = user_token_ref['metadata'].copy()
 
@@ -390,6 +435,11 @@ class Router(object):
           mapper.connect(None, '/v1.0/{path_info:.*}', controller=BlogApp())
 
         """
+        # if we're only running in debug, bump routes' internal logging up a
+        # notch, as it's very spammy
+        if CONF.debug:
+            logging.getLogger('routes.middleware').setLevel(logging.INFO)
+
         self.map = mapper
         self._router = routes.middleware.RoutesMiddleware(self._dispatch,
                                                           self.map)
@@ -416,7 +466,7 @@ class Router(object):
         match = req.environ['wsgiorg.routing_args'][1]
         if not match:
             return render_exception(
-                exception.NotFound('The resource could not be found.'))
+                exception.NotFound(_('The resource could not be found.')))
         app = match['controller']
         return app
 
@@ -512,10 +562,11 @@ def render_response(body=None, status=None, headers=None):
 
 def render_exception(error):
     """Forms a WSGI response based on the current error."""
-    return render_response(status=(error.code, error.title), body={
-        'error': {
-            'code': error.code,
-            'title': error.title,
-            'message': str(error),
-        }
-    })
+    body = {'error': {
+        'code': error.code,
+        'title': error.title,
+        'message': str(error)
+    }}
+    if isinstance(error, exception.AuthPluginException):
+        body['error']['identity'] = error.authentication
+    return render_response(status=(error.code, error.title), body=body)
