@@ -1,3 +1,7 @@
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2012 OpenStack LLC
+#
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -10,4 +14,4 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone.identity.backends.ldap.core import *  # noqa
+from keystone.identity.backends.ldap.core import *
