@@ -14,26 +14,35 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import webob.dec
+
+from keystone.common import config
+from keystone.common import logging
 from keystone.common import serializer
+from keystone.common import utils
 from keystone.common import wsgi
-from keystone import config
 from keystone import exception
 from keystone.openstack.common import jsonutils
 
 
 CONF = config.CONF
+LOG = logging.getLogger(__name__)
 
 
 # Header used to transmit the auth token
 AUTH_TOKEN_HEADER = 'X-Auth-Token'
 
 
+# Header used to transmit the subject token
+SUBJECT_TOKEN_HEADER = 'X-Subject-Token'
+
+
 # Environment variable used to pass the request context
-CONTEXT_ENV = 'openstack.context'
+CONTEXT_ENV = wsgi.CONTEXT_ENV
 
 
 # Environment variable used to pass the request params
-PARAMS_ENV = 'openstack.params'
+PARAMS_ENV = wsgi.PARAMS_ENV
 
 
 class TokenAuthMiddleware(wsgi.Middleware):
@@ -41,6 +50,9 @@ class TokenAuthMiddleware(wsgi.Middleware):
         token = request.headers.get(AUTH_TOKEN_HEADER)
         context = request.environ.get(CONTEXT_ENV, {})
         context['token_id'] = token
+        if SUBJECT_TOKEN_HEADER in request.headers:
+            context['subject_token_id'] = (
+                request.headers.get(SUBJECT_TOKEN_HEADER))
         request.environ[CONTEXT_ENV] = context
 
 
@@ -102,7 +114,7 @@ class JsonBodyMiddleware(wsgi.Middleware):
 
         # Reject unrecognized content types. Empty string indicates
         # the client did not explicitly set the header
-        if not request.content_type in ('application/json', ''):
+        if request.content_type not in ('application/json', ''):
             e = exception.ValidationError(attribute='application/json',
                                           target='Content-Type header')
             return wsgi.render_exception(e)
@@ -137,7 +149,14 @@ class XmlBodyMiddleware(wsgi.Middleware):
         incoming_xml = 'application/xml' in str(request.content_type)
         if incoming_xml and request.body:
             request.content_type = 'application/json'
-            request.body = jsonutils.dumps(serializer.from_xml(request.body))
+            try:
+                request.body = jsonutils.dumps(
+                    serializer.from_xml(request.body))
+            except Exception:
+                LOG.exception('Serializer failed')
+                e = exception.ValidationError(attribute='valid XML',
+                                              target='request body')
+                return wsgi.render_exception(e)
 
     def process_response(self, request, response):
         """Transform the response from JSON to XML."""
@@ -147,7 +166,8 @@ class XmlBodyMiddleware(wsgi.Middleware):
             try:
                 body_obj = jsonutils.loads(response.body)
                 response.body = serializer.to_xml(body_obj)
-            except:
+            except Exception:
+                LOG.exception('Serializer failed')
                 raise exception.Error(message=response.body)
         return response
 
@@ -164,3 +184,21 @@ class NormalizingFilter(wsgi.Middleware):
         # Rewrites path to root if no path is given.
         elif not request.environ['PATH_INFO']:
             request.environ['PATH_INFO'] = '/'
+
+
+class RequestBodySizeLimiter(wsgi.Middleware):
+    """Limit the size of an incoming request."""
+
+    def __init__(self, *args, **kwargs):
+        super(RequestBodySizeLimiter, self).__init__(*args, **kwargs)
+
+    @webob.dec.wsgify(RequestClass=wsgi.Request)
+    def __call__(self, req):
+
+        if req.content_length > CONF.max_request_body_size:
+            raise exception.RequestTooLarge()
+        if req.content_length is None and req.is_body_readable:
+            limiter = utils.LimitingReader(req.body_file,
+                                           CONF.max_request_body_size)
+            req.body_file = limiter
+        return self.application
