@@ -1,4 +1,6 @@
-# Copyright 2012 OpenStack Foundation
+# vim: tabstop=4 shiftwidth=4 softtabstop=4
+
+# Copyright 2012 OpenStack LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -12,5 +14,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone.policy import controllers  # noqa
-from keystone.policy.core import *  # noqa
+from keystone.policy import controllers
+from keystone.policy.core import *
+from keystone.policy import routers
