@@ -63,9 +63,11 @@ class Token(token.Driver):
 
     def create_token(self, token_id, data):
         data_copy = copy.deepcopy(data)
-        ptk = self._prefix_token_id(self.token_to_key(token_id))
-        if 'expires' not in data_copy:
-            data_copy['expires'] = self._get_default_expire_time()
+        ptk = self._prefix_token_id(token.unique_id(token_id))
+        if not data_copy.get('expires'):
+            data_copy['expires'] = token.default_expire_time()
+        if not data_copy.get('user_id'):
+            data_copy['user_id'] = data_copy['user']['id']
         kwargs = {}
         if data_copy['expires'] is not None:
             expires_ts = utils.unixtime(data_copy['expires'])
@@ -93,13 +95,13 @@ class Token(token.Driver):
 
     def delete_token(self, token_id):
         # Test for existence
-        data = self.get_token(self.token_to_key(token_id))
-        ptk = self._prefix_token_id(self.token_to_key(token_id))
+        data = self.get_token(token.unique_id(token_id))
+        ptk = self._prefix_token_id(token.unique_id(token_id))
         result = self.client.delete(ptk)
         self._add_to_revocation_list(data)
         return result
 
-    def list_tokens(self, user_id, tenant_id=None):
+    def list_tokens(self, user_id, tenant_id=None, trust_id=None):
         tokens = []
         user_key = self._prefix_user_id(user_id)
         user_record = self.client.get(user_key) or ""
@@ -114,6 +116,13 @@ class Token(token.Driver):
                         continue
                     if tenant.get('id') != tenant_id:
                         continue
+                if trust_id is not None:
+                    trust = token_ref.get('trust_id')
+                    if not trust:
+                        continue
+                    if trust != trust_id:
+                        continue
+
                 tokens.append(token_id)
         return tokens
 
