@@ -14,44 +14,53 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import re
+import copy
+import StringIO
+
+from lxml import etree
 
 from keystone.common import serializer
 from keystone import test
 
 
 class XmlSerializerTestCase(test.TestCase):
-    def assertEqualIgnoreWhitespace(self, a, b):
-        """Splits two strings into lists and compares them.
+    def assertEqualXML(self, a, b):
+        """Parses two XML documents from strings and compares the results.
 
         This provides easy-to-read failures from nose.
 
         """
-        try:
-            self.assertEqual(a, b)
-        except:
-            a = re.sub('[ \n]+', ' ', a).strip().split()
-            b = re.sub('[ \n]+', ' ', b).strip().split()
-            self.assertEqual(a, b)
+        parser = etree.XMLParser(remove_blank_text=True)
+
+        def canonical_xml(s):
+            s = s.strip()
+
+            fp = StringIO.StringIO()
+            dom = etree.fromstring(s, parser)
+            dom.getroottree().write_c14n(fp)
+            s = fp.getvalue()
+
+            dom = etree.fromstring(s, parser)
+            return etree.tostring(dom, pretty_print=True)
+
+        a = canonical_xml(a)
+        b = canonical_xml(b)
+        self.assertEqual(a.split('\n'), b.split('\n'))
 
     def assertSerializeDeserialize(self, d, xml, xmlns=None):
-        self.assertEqualIgnoreWhitespace(serializer.to_xml(d, xmlns), xml)
+        self.assertEqualXML(
+            serializer.to_xml(copy.deepcopy(d), xmlns),
+            xml)
         self.assertEqual(serializer.from_xml(xml), d)
 
-        # operations should be invertable
+        # operations should be invertible
         self.assertEqual(
-            serializer.from_xml(serializer.to_xml(d, xmlns)),
+            serializer.from_xml(serializer.to_xml(copy.deepcopy(d), xmlns)),
             d)
-        self.assertEqualIgnoreWhitespace(
+        self.assertEqualXML(
             serializer.to_xml(serializer.from_xml(xml), xmlns),
             xml)
 
-    def test_none(self):
-        d = None
-        xml = None
-
-        self.assertSerializeDeserialize(d, xml)
-
     def test_auth_request(self):
         d = {
             "auth": {
@@ -100,9 +109,7 @@ class XmlSerializerTestCase(test.TestCase):
         xmlns = "http://docs.openstack.org/identity/api/ext/OS-KSADM/v1.0"
 
         d = {
-            # FIXME(dolph): should be...
-            # "OS-KSADM:service": {
-            "service": {
+            "OS-KSADM:service": {
                 "id": "123",
                 "name": "nova",
                 "type": "compute",
@@ -147,6 +154,17 @@ class XmlSerializerTestCase(test.TestCase):
 
         self.assertSerializeDeserialize(d, xml)
 
+    def test_policy_list(self):
+        d = {"policies": [{"id": "ab12cd"}]}
+
+        xml = """
+            <?xml version="1.0" encoding="UTF-8"?>
+            <policies xmlns="http://docs.openstack.org/identity/api/v2.0">
+                <policy id="ab12cd"/>
+            </policies>
+        """
+        self.assertEqualXML(serializer.to_xml(d), xml)
+
     def test_values_list(self):
         d = {
             "objects": {
@@ -166,4 +184,72 @@ class XmlSerializerTestCase(test.TestCase):
             </objects>
         """
 
-        self.assertEqualIgnoreWhitespace(serializer.to_xml(d), xml)
+        self.assertEqualXML(serializer.to_xml(d), xml)
+
+    def test_collection_list(self):
+        d = {
+            "links": {
+                "next": "http://localhost:5000/v3/objects?page=3",
+                "previous": None,
+                "self": "http://localhost:5000/v3/objects"
+            },
+            "objects": [{
+                "attribute": "value1",
+                "links": {
+                    "self": "http://localhost:5000/v3/objects/abc123def",
+                    "anotherobj": "http://localhost:5000/v3/anotherobjs/123"
+                }
+            }, {
+                "attribute": "value2",
+                "links": {
+                    "self": "http://localhost:5000/v3/objects/abc456"
+                }
+            }]}
+        xml = """
+            <?xml version="1.0" encoding="UTF-8"?>
+            <objects xmlns="http://docs.openstack.org/identity/api/v2.0">
+                <object attribute="value1">
+                    <links>
+                        <link rel="self"
+                            href="http://localhost:5000/v3/objects/abc123def"/>
+                        <link rel="anotherobj"
+                            href="http://localhost:5000/v3/anotherobjs/123"/>
+                    </links>
+                </object>
+                <object attribute="value2">
+                     <links>
+                         <link rel="self"
+                             href="http://localhost:5000/v3/objects/abc456"/>
+                     </links>
+                </object>
+                <links>
+                    <link rel="self"
+                        href="http://localhost:5000/v3/objects"/>
+                    <link rel="next"
+                        href="http://localhost:5000/v3/objects?page=3"/>
+                </links>
+            </objects>
+        """
+        self.assertSerializeDeserialize(d, xml)
+
+    def test_collection_member(self):
+        d = {
+            "object": {
+                "attribute": "value",
+                "links": {
+                    "self": "http://localhost:5000/v3/objects/abc123def",
+                    "anotherobj": "http://localhost:5000/v3/anotherobjs/123"}}}
+
+        xml = """
+            <?xml version="1.0" encoding="UTF-8"?>
+            <object xmlns="http://docs.openstack.org/identity/api/v2.0"
+                attribute="value">
+                    <links>
+                        <link rel="self"
+                            href="http://localhost:5000/v3/objects/abc123def"/>
+                        <link rel="anotherobj"
+                            href="http://localhost:5000/v3/anotherobjs/123"/>
+                    </links>
+            </object>
+        """
+        self.assertSerializeDeserialize(d, xml)
