@@ -29,10 +29,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import datetime
-
-from keystone import test
 from keystone.common import utils
+from keystone import test
 
 
 class UtilsTestCase(test.TestCase):
@@ -60,13 +58,6 @@ class UtilsTestCase(test.TestCase):
         self.assertTrue(utils.check_password(password, hashed))
         self.assertFalse(utils.check_password(wrong, hashed))
 
-    def test_isotime(self):
-        dt = datetime.datetime(year=1987, month=10, day=13,
-                               hour=1, minute=2, second=3)
-        output = utils.isotime(dt)
-        expected = '1987-10-13T01:02:03Z'
-        self.assertEqual(output, expected)
-
     def test_auth_str_equal(self):
         self.assertTrue(utils.auth_str_equal('abc123', 'abc123'))
         self.assertFalse(utils.auth_str_equal('a', 'aaaaa'))
